<?php

namespace Drupal\livehelperchat\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class liveHelperChatFormSettings.
 */
class liveHelperChatFormSettings extends ConfigFormBase {

  /**
   * Drupal\Core\Database\Driver\mysql\Connection definition.
   *
   * @var \Drupal\Core\Database\Driver\mysql\Connection
   */
  protected $database;

  /**
   * Drupal\Core\Routing\CurrentRouteMatch definition.
   *
   * @var \Drupal\Core\Routing\CurrentRouteMatch
   */
  protected $currentRouteMatch;

  /**
   * Drupal\Core\Session\AccountProxyInterface definition.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * Drupal\Core\Entity\EntityManagerInterface definition.
   *
   * @var \Drupal\Core\Entity\EntityManagerInterface
   */
  protected $entityManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->database = $container->get('database');
    $instance->currentRouteMatch = $container->get('current_route_match');
    $instance->currentUser = $container->get('current_user');
    $instance->entityManager = $container->get('entity.manager');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'livehelperchat.livehelperchatformsettings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'live_helper_chat_form_settings';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $currentUser = $this->currentUser;
    $config = $this->config('livehelperchat.livehelperchatformsettings');

    $form['#attached']['library'][] = 'livehelperchat/ace-editor';
    $form['livehelperchat_type_settings'] = array(
      '#type' => 'radios',
      '#title' => $this->t('Specify the type of chat settings.'),
      '#options' => array('0'=> $this->t('Configure LiveHelperChat visually by administration.'), '1'=> $this->t('Configure LiveHelperChat using JS script generated by chat server in https://demo.livehelperchat.com/site_admin/system/htmlcode#!#design')),
      '#default_value' => $config->get('livehelperchat_type_settings'),
      '#required' => TRUE,
    );


    /**
     * Option 1: Configure LiveHelperChat visually by administration.
     **/

    $form['livehelperchat'] = array(
      '#type' => 'vertical_tabs',
      '#states' => [
        'visible' => [
          ':input[name="livehelperchat_type_settings"]' => ['value' => 0],
        ],
      ],
    );

    $form['account'] = array(
      '#type' => 'details',
      '#title' => $this->t('General settings'),
      '#collapsible' => FALSE,
      '#group' => 'livehelperchat',
    );
    $form['account']['livehelperchat_url'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Live Helper Chat base URL'),
      '#default_value' => $config->get('livehelperchat_url'),
      '#size' => 40,
      '#maxlength' => 40,
      '#required' => TRUE,
      '#description' => $this->t('The URL should not contain protocol or the tailing slash. Ie. for https://demo.livehelperchat.com use demo.livehelperchat.com'),
    );

    $form['widget'] = array(
      '#type' => 'details',
      '#title' => $this->t('Widget settings'),
      '#collapsible' => FALSE,
      '#group' => 'livehelperchat',
    );
    $form['widget']['livehelperchat_widget_height'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Widget height'),
      '#default_value' => $config->get('livehelperchat_widget_height'),
    );
    $form['widget']['livehelperchat_widget_width'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Widget width'),
      '#default_value' => $config->get('livehelperchat_widget_width'),
    );
    $form['widget']['livehelperchat_popup_height'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Popup height'),
      '#default_value' => $config->get('livehelperchat_popup_height'),
    );
    $form['widget']['livehelperchat_popup_width'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Popup width'),
      '#default_value' => $config->get('livehelperchat_popup_width'),
    );
    $form['widget']['livehelperchat_position_fromtop'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Position from top'),
      '#description' => $this->t('Only used if position Middle left or the Middle right side is chosen'),
      '#default_value' => $config->get('livehelperchat_position_fromtop'),
    );
    $form['widget']['livehelperchat_position_unit'] = array(
      '#type' => 'select',
      '#title' => $this->t('Position from top unit'),
      '#description' => $this->t('Only used if position Middle left or the Middle right side is chosen'),
      '#default_value' => $config->get('livehelperchat_position_unit'),
      '#options' => array(
        'pixels' => $this->t('Pixels'),
        'percents' => $this->t('Percents'),
      ),
    );
    $form['widget']['livehelperchat_click_internal'] = array(
      '#type' => 'checkbox',
      '#title' => $this->t('On a mouse click show the page widget'),
      '#default_value' => $config->get('livehelperchat_click_internal'),
    );
    $form['widget']['livehelperchat_check_messages'] = array(
      '#type' => 'checkbox',
      '#title' => $this->t('Check automatically for messages from the operator'),
      '#default_value' => $config->get('livehelperchat_check_messages'),
    );
    $form['widget']['livehelperchat_disable_proactive'] = array(
      '#type' => 'checkbox',
      '#title' => $this->t('Disable pro active invitations'),
      '#default_value' => $config->get('livehelperchat_disable_proactive'),
    );
    $form['widget']['livehelperchat_hide_offline'] = array(
      '#type' => 'checkbox',
      '#title' => $this->t('Hide when operator is offline'),
      '#default_value' => $config->get('livehelperchat_hide_offline'),
    );
    $form['widget']['livehelperchat_offline_leavemessage'] = array(
      '#type' => 'checkbox',
      '#title' => $this->t('Show a leave message form when operator is offline'),
      '#default_value' => $config->get('livehelperchat_offline_leavemessage'),
    );
    $form['widget']['livehelperchat_nonresponsive_widget'] = array(
      '#type' => 'checkbox',
      '#title' => $this->t('Disable responsive layout of the status widget'),
      '#default_value' => $config->get('livehelperchat_nonresponsive_widget'),
    );
    $form['widget']['livehelperchat_domain'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Domain'),
      '#description' => $this->t('The site base domain, if you want sessions to persist across subdomains.'),
      '#default_value' => $config->get('livehelperchat_domain'),
    );
    $form['widget']['livehelperchat_department'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Department ID'),
      '#default_value' => $config->get('livehelperchat_department'),
    );
    $form['widget']['livehelperchat_theme'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Theme'),
      '#default_value' => $config->get('livehelperchat_theme'),
    );
    $form['widget']['livehelperchat_identifier'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Chat identifier'),
      '#default_value' => $config->get('livehelperchat_identifier'),
    );
    $form['widget']['livehelperchat_minimize_action'] = array(
      '#type' => 'select',
      '#title' => $this->t('Minimize action'),
      '#description' => $this->t('Applies only if status widget is at the bottom'),
      '#options' => array(
        'keep' => $this->t('Keep where it was'),
        'minimize' => $this->t('Minimize to the bottom of the screen'),
      ),
      '#default_value' => $config->get('livehelperchat_minimize_action'),
    );

    $form['widget']['livehelperchat_position'] = array(
      '#type' => 'select',
      '#title' => $this->t('Position'),
      '#options' => array(
        //'original' => $this->t('Native placement - it will be shown where the html is embedded'),
        'bottom_right' => $this->t('Bottom right corner of the screen'),
        'bottom_left' => $this->t('Bottom left corner of the screen'),
        'middle_right' => $this->t('Middle right side of the screen'),
        'middle_left' => $this->t('Middle left side of the screen'),
        'api' => $this->t('Invisible, only JS API will be included'),
      ),
      '#default_value' => $config->get('livehelperchat_position'),
    );


    $form['prefilled_fields'] = array(
      '#type' => 'details',
      '#title' => $this->t('Prefilled fields'),
      '#collapsible' => TRUE,
      '#group' => 'livehelperchat',
    );

    $prefilled_fields = $config->get('livehelperchat_prefilled_fields');
    $i = 0;
    foreach ($prefilled_fields as $name => $value) {
      $form['prefilled_fields']['prefilled_' . $i . '_name'] = array(
        '#type' => 'textfield',
        '#title' => $this->t('Name in LHC'),
        '#default_value' => $name,
      );
      $form['prefilled_fields']['prefilled_' . $i . '_value'] = array(
        '#type' => 'textarea',
        '#title' => $this->t('Prefilled value'),
        '#description' => $this->t('You can also use tokens in the prefilled value.'),
        '#default_value' => $value,
      );
      $i++;
    }
    $form['prefilled_fields']['prefilled_' . $i . '_name'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Name in LHC'),
      '#default_value' => '',
    );
    $form['prefilled_fields']['prefilled_' . $i . '_value'] = array(
      '#type' => 'textarea',
      '#title' => $this->t('Prefilled value'),
      '#description' => $this->t('You can also use tokens in the prefilled value.'),
      '#default_value' => '',
    );
    $form['prefilled_fields']['token_help'] = array(
      '#theme' => 'token_tree_link',
    );

    /***
     * Option 2: Configure LiveHelperChat using JS script generated by chat server in https://demo.livehelperchat.com/site_admin/system/htmlcode#!#design.
     **/

    $form['livehelperchat_codejs'] = array(
      '#type' => 'vertical_tabs',
      '#states' => [
        'visible' => [
          ':input[name="livehelperchat_type_settings"]' => ['value' => 1],
        ],
      ],
    );

    $allow_injectjs = $currentUser->hasPermission('use injectjs server config for livehelperchat');

    if($allow_injectjs){
      $form['details_script_codejs'] = array(
        '#type' => 'details',
        '#title' => $this->t('Settings'),
        '#collapsible' => FALSE,
        '#group' => 'livehelperchat_codejs',
      );
      $form['details_script_codejs']['script_codejs'] = array(
        '#type' => 'textarea',
        '#title' => $this->t('Code JS'),
        '#default_value' => $config->get('script_codejs'),
        '#rows' => 17,
        '#attributes' => array('data-ace-mode' => "javascript"),
        '#prefix' => '<div>',
        '#suffix' => '<div class="resizable"><div class="ace-editor"></div></div></div>',
        "#states" => [
          "required" => [
            ':input[name="livehelperchat_type_settings"]' => ["value" => 1]
          ],
          "visible" => [
            ':input[name="livehelperchat_type_settings"]' => ["value" => 1]
          ]
        ],
        '#description' => $this->t("Enter the JS code generated at https://demo.livehelperchat.com/site_admin/system/htmlcode#!#design without the <script> </script> tags."),
      );
    }


    /***
     * General chat visibility settings
     */

    $form['livehelperchat_visibility_config'] = array(
      '#type' => 'vertical_tabs',
      '#title' => $this->t("General chat visibility settings"),
    );
    $form['role_vis_settings'] = array(
      '#type' => 'details',
      '#title' => $this->t('Roles'),
      '#collapsible' => TRUE,
      '#group' => 'livehelperchat_visibility_config',
    );

    $roles = user_roles();
    $role_options = array();
    foreach ($roles as $rid => $name) {
      $role_options[$rid] = $name->get('label');
    }
    $form['role_vis_settings']['livehelperchat_roles'] = array(
      '#type' => 'checkboxes',
      '#title' => $this->t('Remove script for specific roles'),
      '#default_value' => $config->get('livehelperchat_roles'),
      '#options' => $role_options,
      '#description' => $this->t('Remove script only for the selected role(s). If none of the roles are selected, all roles will have the script. Otherwise, any roles selected here will NOT have the script.'),
    );

    $form['content_type_vis_settings'] = array(
      '#type' => 'details',
      '#title' => $this->t('Content type'),
      '#collapsible' => TRUE,
      '#group' => 'livehelperchat_visibility_config',
    );

    $content_type_options = livehelperchat_list_content_type(FALSE);
    $form['content_type_vis_settings']['livehelperchat_content_type'] = array(
      '#type' => 'checkboxes',
      '#title' => $this->t('Remove script for specific content type'),
      '#default_value' => $config->get('livehelperchat_content_type'),
      '#options' => $content_type_options,
      '#description' => $this->t('Remove script only for the selected content type. If none of the content type are selected, all content type will have the script. Otherwise, any content type selected here will NOT have the script.'),
    );


    $form['page_vis_settings'] = array(
      '#type' => 'details',
      '#title' => $this->t('Pages'),
      '#collapsible' => TRUE,
      '#group' => 'livehelperchat_visibility_config',
    );

    $access = $currentUser->hasPermission('use PHP for livehelperchat visibility');
    $visibility = $config->get('livehelperchat_visibility');
    $pages = $config->get('livehelperchat_pages');


    if ($visibility == 2 && !$access) {
      $form['page_vis_settings'] = array();
      $form['page_vis_settings']['visibility'] = array('#type' => 'value', '#value' => 2);
      $form['page_vis_settings']['pages'] = array('#type' => 'value', '#value' => $pages);
    }
    else {
      $options = array($this->t('Add to every page except the listed pages.'), $this->t('Add to the listed pages only.'));
      $description = $this->t("Enter one page per line as Drupal paths. The '*' character is a wildcard. Example paths are %blog for the blog page and %blog-wildcard for every personal blog. %front is the front page.", array(
        '%blog' => 'blog',
        '%blog-wildcard' => 'blog/*',
        '%front' => '<front>',
      ));
      if ($access) {
        $options[] = $this->t('Add if the following PHP code returns <code>TRUE</code> (PHP-mode, experts only).');
        $description .= ' ' . $this->t('If the PHP-mode is chosen, enter PHP code between %php tags. Note that executing incorrect PHP-code can break your Drupal site.', array('%php' => '<?php ?>'));
      }
      $form['page_vis_settings']['livehelperchat_visibility'] = array(
        '#type' => 'radios',
        '#title' => $this->t('Add script to specific pages'),
        '#options' => $options,
        '#default_value' => $visibility,
      );
      $form['page_vis_settings']['livehelperchat_pages'] = array(
        '#type' => 'textarea',
        '#title' => $this->t('Pages'),
        '#default_value' => (empty($pages))? "" : implode("\n", $pages),
        '#description' => $description,
        '#wysiwyg' => FALSE,
      );
    }


    return parent::buildForm($form, $form_state);
  }


  /**
   * Implements \Drupal\Core\Form\FormInterface::validateForm().
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {

  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);

    $livehelperchat_pages = $form_state->getValue('livehelperchat_pages');
    $livehelperchat_pages = (!empty($livehelperchat_pages))? explode("\n", $livehelperchat_pages): [];
    $allow_injectjs = $this->currentUser->hasPermission('use injectjs server config for livehelperchat');
    $config = $this->config('livehelperchat.livehelperchatformsettings');
    if($allow_injectjs){
      $script_codejs = $form_state->getValue('script_codejs');
    }
    else{
      $script_codejs = $config->get('script_codejs');
    }


    $this->config('livehelperchat.livehelperchatformsettings')
      ->set('livehelperchat_type_settings', $form_state->getValue('livehelperchat_type_settings'))
      ->set('livehelperchat_url', $form_state->getValue('livehelperchat_url'))
      ->set('livehelperchat_widget_height', $form_state->getValue('livehelperchat_widget_height'))
      ->set('livehelperchat_widget_width', $form_state->getValue('livehelperchat_widget_width'))
      ->set('livehelperchat_popup_height', $form_state->getValue('livehelperchat_popup_height'))
      ->set('livehelperchat_popup_width', $form_state->getValue('livehelperchat_popup_width'))
      ->set('livehelperchat_position_fromtop', $form_state->getValue('livehelperchat_position_fromtop'))
      ->set('livehelperchat_position_unit', $form_state->getValue('livehelperchat_position_unit'))
      ->set('livehelperchat_click_internal', $form_state->getValue('livehelperchat_click_internal'))
      ->set('livehelperchat_check_messages', $form_state->getValue('livehelperchat_check_messages'))
      ->set('livehelperchat_disable_proactive', $form_state->getValue('livehelperchat_disable_proactive'))
      ->set('livehelperchat_hide_offline', $form_state->getValue('livehelperchat_hide_offline'))
      ->set('livehelperchat_offline_leavemessage', $form_state->getValue('livehelperchat_offline_leavemessage'))
      ->set('livehelperchat_nonresponsive_widget', $form_state->getValue('livehelperchat_nonresponsive_widget'))
      ->set('livehelperchat_domain', $form_state->getValue('livehelperchat_domain'))
      ->set('livehelperchat_department', $form_state->getValue('livehelperchat_department'))
      ->set('livehelperchat_theme', $form_state->getValue('livehelperchat_theme'))
      ->set('livehelperchat_identifier', $form_state->getValue('livehelperchat_identifier'))
      ->set('livehelperchat_minimize_action', $form_state->getValue('livehelperchat_minimize_action'))
      ->set('livehelperchat_position', $form_state->getValue('livehelperchat_position'))
      ->set('livehelperchat_roles', $form_state->getValue('livehelperchat_roles'))
      ->set('livehelperchat_content_type', $form_state->getValue('livehelperchat_content_type'))
      ->set('livehelperchat_visibility', $form_state->getValue('livehelperchat_visibility'))
      ->set('livehelperchat_pages', $livehelperchat_pages)
      ->set('script_codejs', $script_codejs)
      ->save();
  }

}
